@@ -25,3 +25,11 @@ class DuplicateExtensionException(Exception):
     def __init__(self, message: str = "Duplicate extension not allowed") -> None:
         self.message = message
         super().__init__(self.message)
+
+
+class OCSPMissingExtensionException(Exception):
+    """Class to handle OCSP missing extension exceptions"""
+
+    def __init__(self, message: str = "Required OCSP extension not found") -> None:
+        self.message = message
+        super().__init__(self.message)
