@@ -10,6 +10,7 @@ Exposes the functions:
 - key_labels()
 - sign()
 - verify()
+- delete_keypair()
 - public_key_data()
 """
 
@@ -21,6 +22,7 @@ from concurrent.futures import ThreadPoolExecutor
 from asyncio import get_event_loop, sleep
 from contextlib import asynccontextmanager
 
+import pkcs11
 from asn1crypto import pem as asn1_pem
 from asn1crypto.keys import (
     OctetString,
@@ -37,7 +39,6 @@ from pkcs11 import (
     lib,
     Mechanism,
     ObjectClass,
-    PublicKey,
     Session,
     Token,
 )
@@ -50,6 +51,7 @@ from pkcs11.util.ec import (
     decode_ec_private_key,
 )
 
+from .crypto import convert_asn1_ec_signature, convert_rs_ec_signature
 from .error import PKCS11TimeoutException, PKCS11UnknownErrorException
 from .lib import DEBUG, key_types, key_type_values
 
@@ -57,58 +59,6 @@ TIMEOUT = 3  # Seconds
 pool = ThreadPoolExecutor()
 
 
-def convert_ec_signature_openssl_format(signature: bytes, key_type: str) -> bytes:
-    """Convert an R&S ECDSA signature into the default openssl format.
-
-    https://stackoverflow.com/questions/66101825/asn-1-structure-of-ecdsa-signature-in-x-509-certificate
-
-    Paramters:
-    signature (bytes): The signature.
-    key_type (str): Key type.
-
-    Returns:
-    bytes
-    """
-
-    asn1_integer_code = 2
-    asn1_init = bytearray([48])
-
-    if key_type in ["secp521r1"]:
-        asn1_init.append(129)
-
-    r_length = int(len(signature) / 2)
-    s_length = int(len(signature) / 2)
-
-    r_data = signature[:r_length]
-    s_data = signature[r_length:]
-
-    # Remove leading zeros, since integers cant start with a 0
-    while r_data[0] == 0:
-        r_data = r_data[1:]
-        r_length -= 1
-    while s_data[0] == 0:
-        s_data = s_data[1:]
-        s_length -= 1
-
-    # Ensure the integers are postive numbers
-    if r_data[0] >= 128:
-        r_data = bytearray([0]) + r_data[:]
-        r_length += 1
-
-    if s_data[0] >= 128:
-        s_data = bytearray([0]) + s_data[:]
-        s_length += 1
-
-    return bytes(
-        asn1_init
-        + bytearray([r_length + s_length + 4])
-        + bytearray([asn1_integer_code, r_length])
-        + r_data
-        + bytearray([asn1_integer_code, s_length])
-        + s_data
-    )
-
-
 # Taken from https://github.com/danni/python-pkcs11/blob/master/pkcs11/util/ec.py
 # Will submit merge request soon
 def decode_eddsa_public_key(der: bytes, encode_eddsa_point: bool = True) -> Dict[int, Union[str, int, bytes]]:
@@ -163,7 +113,7 @@ def decode_eddsa_private_key(der: bytes) -> Dict[int, Union[str, int, bytes]]:
     }
 
 
-def encode_eddsa_public_key(key: PublicKey) -> bytes:
+def encode_eddsa_public_key(key: pkcs11.Key) -> bytes:
     """
     Encode a DER-encoded EdDSA public key as stored by OpenSSL.
     :param PublicKey key: EdDSA public key
@@ -200,15 +150,8 @@ class PKCS11Session:
     session: Union[Session, None] = None
     token: Union[Token, None] = None
 
-    # def __del__(cls) -> None:
-    # print("removing")
-    # cls.session.close()
-
     @classmethod
     def _close_session(cls) -> None:
-        # print("closing")
-        # pkcs11_lib = lib(os.environ["PKCS11_MODULE"])
-        # cls.token = pkcs11_lib.get_token(token_label=os.environ["PKCS11_TOKEN"])
         if cls.session is not None:
             cls.session.close()
             cls.session = None
@@ -219,10 +162,10 @@ class PKCS11Session:
         try:
             # if force or cls.session is None:
             if force or cls.session is None:
-                # print("gg1")
                 pkcs11_lib = lib(os.environ["PKCS11_MODULE"])
                 pkcs11_lib.reinitialize()
                 cls.token = pkcs11_lib.get_token(token_label=os.environ["PKCS11_TOKEN"])
+                # user_pin need to be a string, not bytes
                 cls.session = cls.token.open(rw=True, user_pin=os.environ["PKCS11_PIN"])
 
             _ = cls.session.get_key(
@@ -244,7 +187,6 @@ class PKCS11Session:
             if DEBUG:
                 print("Token failed")
                 print(exc)
-            # raise exc
 
     @classmethod
     async def _healthy_session(cls) -> None:
@@ -254,8 +196,7 @@ class PKCS11Session:
 
         if thread.is_alive() or cls._session_status != 0:
             if DEBUG:
-                # print(cls._session_status)
-                # print(thread.is_alive())
+
                 print("Current PKCS11 session is unhealthy, opening a new session")
 
             thread2 = Thread(target=cls._close_session, args=())
@@ -592,7 +533,7 @@ class PKCS11Session:
 
         # PKCS11 specific stuff for EC curves, sig is in R&S format, convert it to openssl format
         if key_type in ["secp256r1", "secp384r1", "secp521r1"]:
-            signature = convert_ec_signature_openssl_format(signature, key_type)
+            signature = convert_rs_ec_signature(signature, key_type)
 
         return signature
 
@@ -656,6 +597,12 @@ class PKCS11Session:
                 hash_obj.update(data)
                 data = hash_obj.digest()
 
+                try:
+                    signature = convert_asn1_ec_signature(signature, key_type)
+                except (IndexError, ValueError):
+                    # Signature was not in ASN1 format, signature verification will probably fail.
+                    pass
+
             else:  # rsa
                 if key_type == "rsa_2048":
                     mech = Mechanism.SHA256_RSA_PKCS
@@ -666,6 +613,44 @@ class PKCS11Session:
                 return True
             return False
 
+    @classmethod
+    async def delete_keypair(cls, key_label: str, key_type: Union[str, None] = None) -> None:
+        """Delete the keypair from the PKCS11 device.
+
+        Parameters:
+        key_label (str): Keypair label.
+        key_type (Union[str, None] = None): Key type.
+
+        Returns:
+        None
+        """
+
+        if key_type is None:
+            key_type = "ed25519"
+
+        if key_type not in key_types:
+            raise ValueError(f"key_type must be in {key_types}")
+
+        async with async_lock(cls._lock):
+            # Ensure we get a healthy pkcs11 session
+            await cls._healthy_session()
+
+            if cls.session is None:
+                raise PKCS11UnknownErrorException
+
+            try:
+                cls.session.get_key(
+                    key_type=key_type_values[key_type],
+                    object_class=ObjectClass.PUBLIC_KEY,
+                    label=key_label,
+                ).destroy()
+            finally:
+                cls.session.get_key(
+                    key_type=key_type_values[key_type],
+                    object_class=ObjectClass.PRIVATE_KEY,
+                    label=key_label,
+                ).destroy()
+
     @classmethod
     async def public_key_data(cls, key_label: str, key_type: Union[str, None] = None) -> Tuple[str, bytes]:
         """Returns the public key in PEM form
